-- up_out.vhd
-- by Toshio Iwata at digitalfilter.com 2009/06/12

library ieee;
use STD.TEXTIO.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;
use ieee.std_logic_arith.all;
use work.globals_pkg.all;

entity up_out is
  Port (
  up_addr : out std_logic_vector(9 downto 0);
  up_wr_data : out std_logic_vector(15 downto 0);
  up_we_n : out std_logic;
  up_oe_n : out std_logic;
  up_imag_sel : out std_logic;
  FIFO_RD_START : out std_logic;
  FIFO_WR_START : out std_logic;
  iW_ce0 : in std_logic;
  iW_data : out std_logic_vector(15 downto 0);
  CAP : out std_logic;
  FFT : out std_logic;
  RST : out std_logic );
end up_out;

architecture beh of up_out is

begin

gen_RST : process
begin
   RST <= '0';
   wait for 5 us;
   RST <= '1';
   wait;
end process gen_RST;

process
begin
   CAP <= '0';
   wait for 1 ms;
   if(sv_testmode) then
     CAP <= '0';
   else 
     CAP <= '1';
   end if;
   wait for 19 ms;
   CAP <= '0';
   wait;
end process;

process
begin
   FFT <= '0';
   wait for 800 us;
   FFT <= '1';
   wait for 1 ms;
   FFT <= '0';
   wait;
end process;

gen_up_addr : process
variable linecount : integer := 0;
variable first_time : boolean := true;
variable addr_tmp : unsigned( 9 downto 0 );
begin
  if( first_time ) then
    up_addr <= "0000000000";
    wait for 500 us;
    first_time := false;
  end if;
  if( sv_testmode and linecount < 512 ) then 
    addr_tmp := conv_unsigned(linecount, 10);
    for i in 0 to 8 loop
      up_addr(8-i) <= addr_tmp(i); -- iwata
    end loop;
    wait for sv_fsclk_period;
    linecount := linecount + 1;
  else
    up_addr <= (others => '0');
    wait for sv_fsclk_period;
  end if;
end process;

process
variable Li : line;
variable linecount : integer := 0;
variable first_time : boolean := true;
variable charbuf       : character ;
begin
  wait until iW_ce0 = '1';
  if( sv_testmode and linecount < 512 ) then 
    readline(sv_file_in, Li);
    for i in 0 to 15 loop
      read(Li, charbuf);
      if ( charbuf = '0' ) then
        iW_data(15-i) <= '0';
      else 
        iW_data(15-i) <= '1';
      end if;
    end loop;
    linecount := linecount + 1;
  else
    iW_data <= (others => '0');
  end if;
end process;

process
begin
   up_we_n <= '1';
   FIFO_RD_START <= '0';
   wait for 500 us;
   if(sv_testmode) then
     up_we_n <= '0';
     FIFO_RD_START <= '1';
   else 
     up_we_n <= '1';
     FIFO_RD_START <= '0';
   end if;
   wait for (160 ns * 512.0);
   up_we_n <= '1';
   FIFO_RD_START <= '0';
   wait;
end process;

process
begin
   FIFO_WR_START <= '0';
   wait for 1100 us;
   if(sv_testmode) then
     FIFO_WR_START <= '1';
   else 
     FIFO_WR_START <= '0';
   end if;
   wait for (160 ns * 512.0);
   FIFO_WR_START <= '0';
   wait;
end process;

process
begin
   up_oe_n <= '1';
   wait for 1100 us;
   up_oe_n <= '0';
   wait for 1 ms;
   up_oe_n <= '1';
   wait;
end process;

process
begin
   up_imag_sel <= '0';
   wait for 18 ms;
   up_imag_sel <= '0';
   wait for 1 us;
   up_imag_sel <= '0';
   wait for 1 us;
   up_imag_sel <= '1';
   wait for 1 us;
   up_imag_sel <= '1';
   wait;
end process;

end beh;
