-------------------------------
-- RamCtrl.vhd
-- Initial DF/F to latch serial data
-- by Toshio Iwata at digitalfilter.com 2009/06/12

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;
  use ieee.std_logic_signed.all ;

entity RamCtrl is
   Port (
    MCLK         : in    std_logic                           ; 
    RST_X        : in    std_logic                           ; 
  FIFO_RD_START : in std_logic;                      -- a trigger to read fifo
  FIFO_WR_START : in std_logic;                      -- a trigger to write fifo
  iW_ce0 : out std_logic;
  iW_data : in std_logic_vector( 31 downto 0 )  ; 
  oR_ce0 : out std_logic;
  oR_data : out std_logic_vector( 31 downto 0 )  ; 
    FFT_ADDR     : in    std_logic_vector(   9 downto   0 )  ; 
    CAP_SEL      : in    std_logic                           ; 
    FFT_SEL      : in    std_logic                           ; 
    REAL_RD_ADDR : out   std_logic_vector(   9 downto   0 )  ; 
    IMAG_RD_ADDR : out   std_logic_vector(   9 downto   0 )  ; 
    REAL_WR_ADDR : out   std_logic_vector(   9 downto   0 )  ; 
    IMAG_WR_ADDR : out   std_logic_vector(   9 downto   0 )  ; 
    PHASE_I : in std_logic;
    CS_0_X    : out   std_logic                           ; 
    CS_1_X    : out   std_logic                           ; 
    WR_0_X    : out   std_logic                           ; 
    WR_1_X    : out   std_logic                           ; 
    REAL_RD_DATA : in    std_logic_vector(  15 downto   0 )  ; 
    IMAG_RD_DATA : in    std_logic_vector(  15 downto   0 )  ;
    REAL_WR_DATA : out   std_logic_vector(  15 downto   0 )  ;
    IMAG_WR_DATA : out   std_logic_vector(  15 downto   0 )  ; 
    FFT_REAL_RD_DATA : out   std_logic_vector(  15 downto   0 )  ;
    FFT_IMAG_RD_DATA : out   std_logic_vector(  15 downto   0 )  ; 
    FFT_REAL_WR_DATA : in    std_logic_vector(  15 downto   0 )  ;
    FFT_IMAG_WR_DATA : in    std_logic_vector(  15 downto   0 )  ; 
    ADC_ADDR : in std_logic_vector(   9 downto   0 )  ;
    ADC_DATA : in std_logic_vector(  15 downto   0 )  ;
  UP_IMAG_SEL : In std_logic );
end RamCtrl;

architecture STRUCTURE of RamCtrl is

  signal   r_fft_sel_dly1       : std_logic                           ; 
  signal   r_fft_sel_dly2       : std_logic                           ;
  signal   r_fft_sel_dly3       : std_logic                           ; 
  signal   r_fft_sel_dly4       : std_logic                           ; 
  signal   r_fft_sel_dly5       : std_logic                           ;
  signal   r_fft_sel_dly6       : std_logic                           ; 
  signal   r_fft_addr_dly1      : std_logic_vector(   9 downto   0 )  ; 
  signal   r_fft_addr_dly2      : std_logic_vector(   9 downto   0 )  ; 
  signal   r_fft_addr_dly3      : std_logic_vector(   9 downto   0 )  ; 
  signal   r_fft_addr_dly4      : std_logic_vector(   9 downto   0 )  ; 
  signal   r_fft_addr_dly5      : std_logic_vector(   9 downto   0 )  ;
  signal   r_fft_addr_dly6      : std_logic_vector(   9 downto   0 )  ; 
  signal   phase_i_dly1      : std_logic                           ;
  signal   phase_i_dly2      : std_logic                           ; 
  signal   phase_i_dly3      : std_logic                           ; 
  signal   phase_i_dly4      : std_logic                           ; 
  signal   phase_i_dly5      : std_logic                           ;
  signal   phase_i_dly6      : std_logic                           ;
	signal FIFO_RD_START_dly1 : std_logic;
	signal FIFO_RD_START_dly2 : std_logic;
	signal FIFO_RD_START_dly3 : std_logic;
	signal FIFO_RD_START_edge : std_logic;
	signal FIFO_WR_START_dly1 : std_logic;
	signal FIFO_WR_START_dly2 : std_logic;
	signal FIFO_WR_START_dly3 : std_logic;
	signal FIFO_WR_START_edge : std_logic;
  signal   oR_addr      : std_logic_vector( 9 downto 0 )  ; 
  signal   iW_addr      : std_logic_vector( 9 downto 0 )  ; 
  signal counter_iW : unsigned(11 downto 0);
  signal counter_oR : unsigned(11 downto 0);
  signal oR_ce0_tmp, oR_ce0_dly1, oR_ce0_dly2 : std_logic;

begin -- architecture RamCtrl
-------------------------------------------------------------------
	process( RST_X, MCLK )
	begin
		if( RST_X = '0' ) then
			FIFO_RD_START_dly1 <= '0';
			FIFO_RD_START_dly2 <= '0';
			FIFO_RD_START_dly3 <= '0';
		elsif( MCLK'event and MCLK = '1' ) then
			FIFO_RD_START_dly1 <= FIFO_RD_START;
			FIFO_RD_START_dly2 <= FIFO_RD_START_dly1;
			FIFO_RD_START_dly3 <= FIFO_RD_START_dly2;
		end if;
	end process;

-------------------------------------------------------------------
	FIFO_RD_START_edge <= FIFO_RD_START_dly1 and not FIFO_RD_START_dly2;

-------------------------------------------------------------------
	process( RST_X, MCLK )
	begin
		if( RST_X = '0' ) then
			FIFO_WR_START_dly1 <= '0';
			FIFO_WR_START_dly2 <= '0';
			FIFO_WR_START_dly3 <= '0';
		elsif( MCLK'event and MCLK = '1' ) then
			FIFO_WR_START_dly1 <= FIFO_WR_START;
			FIFO_WR_START_dly2 <= FIFO_WR_START_dly1;
			FIFO_WR_START_dly3 <= FIFO_WR_START_dly2;
		end if;
	end process;

-------------------------------------------------------------------
	FIFO_WR_START_edge <= FIFO_WR_START_dly1 and not FIFO_WR_START_dly2;

-------------------------------------------------------------------
process( RST_X, MCLK )
begin
  if( RST_X = '0' ) then
    counter_iW  <= (others => '1');
  elsif( MCLK'event and MCLK = '1' ) then
    if(FIFO_RD_START_edge = '1') then 
      counter_iW  <= (others => '0');
    elsif(counter_iW /= 4095 ) then 
      counter_iW <= counter_iW + 1; -- up counter
    end if;
  end if;
end process;

  iW_ce0 <= not counter_iW(2);
  iW_addr <= '0' & counter_iW(3) & counter_iW(4) & counter_iW(5) & counter_iW(6)
     & counter_iW(7) & counter_iW(8) & counter_iW(9) & counter_iW(10) & counter_iW(11);

-------------------------------------------------------------------
process( RST_X, MCLK )
begin
  if( RST_X = '0' ) then
    counter_oR  <= (others => '1');
  elsif( MCLK'event and MCLK = '1' ) then
    if(FIFO_WR_START_edge = '1') then 
      counter_oR  <= (others => '0');
    elsif(counter_oR /= 4095 ) then 
      counter_oR <= counter_oR + 1; -- up counter
    end if;
  end if;
end process;

  oR_ce0_tmp <= not counter_oR(2);
  oR_addr <= '0' & counter_oR(11) & counter_oR(10) & counter_oR(9) & counter_oR(8)
     & counter_oR(7) & counter_oR(6) & counter_oR(5) & counter_oR(4) & counter_oR(3);

-------------------------------------------------------------------
  oR_data <= REAL_RD_DATA & IMAG_RD_DATA;
  
-------------------------------------------------------------------
process( RST_X, MCLK )
begin
  if( RST_X = '0' ) then
    oR_ce0_dly2  <= '0';
    oR_ce0_dly1  <= '0';
  elsif( MCLK'event and MCLK = '1' ) then
    oR_ce0_dly2 <= oR_ce0_dly1;
    oR_ce0_dly1 <= oR_ce0_tmp;
  end if;
end process;
  
  oR_ce0 <= oR_ce0_dly2;
  
  ------------------------------------------------------------
  --  Real RAM Read Address 
  ------------------------------------------------------------
  REAL_RD_ADDR <=     FFT_ADDR  when ( FFT_SEL = '1' ) else
                      oR_addr  when ( FIFO_WR_START = '1' ) else
                      (others => '0') ;
  
  ------------------------------------------------------------
  --  Imag RAM Read Address 
  ------------------------------------------------------------
  IMAG_RD_ADDR <=     FFT_ADDR  when ( FFT_SEL = '1' ) else
                      oR_addr  when ( FIFO_WR_START = '1' ) else
                      (others => '0') ;

  ------------------------------------------------------------
  --  Real RAM Write Address 
  ------------------------------------------------------------
  REAL_WR_ADDR <=     ADC_ADDR when CAP_SEL = '1' else 
                      r_fft_addr_dly6    when ( r_fft_sel_dly6 = '1' ) else
                      iW_addr when FIFO_RD_START = '1' else
                      (others => '0') ;

  ------------------------------------------------------------
  --  Imag RAM Write Address 
  ------------------------------------------------------------
  IMAG_WR_ADDR <=     ADC_ADDR when CAP_SEL = '1' else 
                      r_fft_addr_dly6    when ( r_fft_sel_dly6 = '1' ) else
                      iW_addr when FIFO_RD_START = '1' else
                      (others => '0') ;

  ------------------------------------------------------------
  --  Delayed FFT Address 
  ------------------------------------------------------------
  fft_addr_dly1_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_fft_addr_dly1 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
      r_fft_addr_dly1 <= FFT_ADDR ;
    end if ;
  end process fft_addr_dly1_PROC ;

  ------------------------------------------------------------
  --  Delayed FFT Address 
  ------------------------------------------------------------
  fft_addr_dly2_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_fft_addr_dly2 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
      r_fft_addr_dly2 <= r_fft_addr_dly1 ;
    end if ;
  end process fft_addr_dly2_PROC ;

  ------------------------------------------------------------
  --  Delayed FFT Address 
  ------------------------------------------------------------
  fft_addr_dly3_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_fft_addr_dly3 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
      r_fft_addr_dly3 <= r_fft_addr_dly2 ;
    end if ;
  end process fft_addr_dly3_PROC ;

  ------------------------------------------------------------
  --  Delayed FFT Address 
  ------------------------------------------------------------
  fft_addr_dly4_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_fft_addr_dly4 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
      r_fft_addr_dly4 <= r_fft_addr_dly3 ;
    end if ;
  end process fft_addr_dly4_PROC ;

  ------------------------------------------------------------
  --  Delayed FFT Address 
  ------------------------------------------------------------
  fft_addr_dly5_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_fft_addr_dly5 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
      r_fft_addr_dly5 <= r_fft_addr_dly4 ;
    end if ;
  end process fft_addr_dly5_PROC ;

  ------------------------------------------------------------
  --  Delayed FFT Address 
  ------------------------------------------------------------
  fft_addr_dly6_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_fft_addr_dly6 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
      r_fft_addr_dly6 <= r_fft_addr_dly5 ;
    end if ;
  end process fft_addr_dly6_PROC ;

  ------------------------------------------------------------
  --  Real RAM Write Data 
  ------------------------------------------------------------
  REAL_WR_DATA <=     ADC_DATA when CAP_SEL = '1' else 
                      FFT_REAL_WR_DATA when ( r_fft_sel_dly6 = '1' ) else
                      iW_data(31 downto 16) when FIFO_RD_START = '1' else
                      (others => '0') ;

  ------------------------------------------------------------
  --  Imag RAM Write Data 
  ------------------------------------------------------------
  IMAG_WR_DATA <=     (others => '0') when CAP_SEL = '1' else 
                      FFT_IMAG_WR_DATA  when ( r_fft_sel_dly6 = '1' ) else
                      iW_data(15 downto 0) when FIFO_RD_START = '1' else
                      (others => '0') ;

  ------------------------------------------------------------
  phase_i_dly_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      phase_i_dly6 <= '0';
      phase_i_dly5 <= '0';
      phase_i_dly4 <= '0';
      phase_i_dly3 <= '0';
      phase_i_dly2 <= '0';
      phase_i_dly1 <= '0';
    elsif ( MCLK'event and MCLK = '1' ) then
      phase_i_dly6 <= phase_i_dly5;
      phase_i_dly5 <= phase_i_dly4;
      phase_i_dly4 <= phase_i_dly3;
      phase_i_dly3 <= phase_i_dly2;
      phase_i_dly2 <= phase_i_dly1;
      phase_i_dly1 <= PHASE_I;
    end if ;
  end process ;

  ------------------------------------------------------------
  --  RAM0 Write Enable 
  ------------------------------------------------------------
  WR_0_X <= '0' when CAP_SEL = '1' else 
            '0' when r_fft_sel_dly6 = '1' and phase_i_dly6 = '1' else 
            '0' when FIFO_RD_START = '1' else
            '1';

  ------------------------------------------------------------
  --  RAM1 Write Enable 
  ------------------------------------------------------------
  WR_1_X <= '0' when r_fft_sel_dly6 = '1' and phase_i_dly6 = '0' else 
            '1';
  
  -- '0' when CAP_SEL = '1' else 
               --    '0' when r_fft_sel_dly6 = '1' and phase_i_dly6 = '0' else 
               --    '1';
                   
  ------------------------------------------------------------
  --  RAM0 Chip Select 
  ------------------------------------------------------------
  CS_0_X <= '0' when CAP_SEL = '1' else
            '0' when FFT_SEL = '1' or r_fft_sel_dly6 ='1' else 
            '0' when FIFO_RD_START = '1' else 
            '1';

  ------------------------------------------------------------
  --  RAM1 Chip Select 
  ------------------------------------------------------------
  CS_1_X <= -- '0' when CAP_SEL = '1' else
            '0' when FFT_SEL = '1' or r_fft_sel_dly6 ='1' else 
            '0' when FIFO_WR_START = '1' else 
            '1';
            
  ------------------------------------------------------------
  --  Real Data to FFT 
  ------------------------------------------------------------
  FFT_REAL_RD_DATA <=     REAL_RD_DATA  when ( r_fft_sel_dly1 = '1' ) else
                          (others => '0') ;

  
  ------------------------------------------------------------
  --  Imag Data to FFT 
  ------------------------------------------------------------
  FFT_IMAG_RD_DATA <=     IMAG_RD_DATA  when ( r_fft_sel_dly1 = '1' ) else
                          (others => '0') ;


  ------------------------------------------------------------
  --  Delayed FFT Selection 
  ------------------------------------------------------------
  fft_sel_dly1_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_fft_sel_dly1 <= '0' ;
    elsif ( MCLK'event and MCLK = '1' ) then
      r_fft_sel_dly1 <= FFT_SEL ;
    end if ;
  end process fft_sel_dly1_PROC ;

  ------------------------------------------------------------
  --  Delayed FFT Selection 
  ------------------------------------------------------------
  fft_sel_dly2_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_fft_sel_dly2 <= '0' ;
    elsif ( MCLK'event and MCLK = '1' ) then
      r_fft_sel_dly2 <= r_fft_sel_dly1 ;
    end if ;
  end process fft_sel_dly2_PROC ;

  ------------------------------------------------------------
  --  Delayed FFT Selection 
  ------------------------------------------------------------
  fft_sel_dly3_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_fft_sel_dly3 <= '0' ;
    elsif ( MCLK'event and MCLK = '1' ) then
       r_fft_sel_dly3 <= r_fft_sel_dly2 ;
    end if ;
  end process fft_sel_dly3_PROC ;

  ------------------------------------------------------------
  --  Delayed FFT Selection 
  ------------------------------------------------------------
  fft_sel_dly4_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_fft_sel_dly4 <= '0' ;
    elsif ( MCLK'event and MCLK = '1' ) then
      r_fft_sel_dly4 <= r_fft_sel_dly3 ;
    end if ;
  end process fft_sel_dly4_PROC ;

  ------------------------------------------------------------
  --  Delayed FFT Selection 
  ------------------------------------------------------------
  fft_sel_dly5_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_fft_sel_dly5 <= '0' ;
    elsif ( MCLK'event and MCLK = '1' ) then
      r_fft_sel_dly5 <= r_fft_sel_dly4 ;
    end if ;
  end process fft_sel_dly5_PROC ;

  ------------------------------------------------------------
  --  Delayed FFT Selection 
  ------------------------------------------------------------
  fft_sel_dly6_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_fft_sel_dly6 <= '0' ;
    elsif ( MCLK'event and MCLK = '1' ) then
      r_fft_sel_dly6 <= r_fft_sel_dly5 ;
    end if ;
  end process fft_sel_dly6_PROC ;


end; -- architecture RamCtrl
