#include <stdio.h>
#include <cv.h>
#include <highgui.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdint.h>
#include <string.h>

#define WAVEN 512
#define WIDTH 512
 
void _wait(loop_count)
int loop_count;
{
  volatile int sum, data;
  sum = 0;
  for(data = 0; data < loop_count; data++) {
    sum = (data << 8);
  }
  return;
}

void FifoCtrlWrite(int fd, unsigned char enb);
void allwrite(int fd, unsigned char *buf, int len);
void allread(int fd, unsigned char *buf, int len);
void TxtSave(char *fn, short *iW, int lineno);
void TxtLoad(char *fn, short *iW, int pixno);

int main(int argc, char *argv[]) {
 
  int fdw, fdffr, fdffw, i;
  unsigned char buf[WAVEN*4];
  short inR[WAVEN];
  short inI[WAVEN];
  short fReal[WAVEN];
  short fImag[WAVEN];
  short fAbs[WAVEN];
  short tmpshort;
  long tmplong;
  unsigned char pix[WIDTH];
  int x, y;
  IplImage *result;
  IplImage *srcReal, *srcImag;
  double tmpdouble;

  fdw = open("/dev/xillybus_mem_8", O_WRONLY);

  fdffr = open("/dev/xillybus_read_32", O_RDONLY);
  fdffw = open("/dev/xillybus_write_32", O_WRONLY);
 
  if (fdffr < 0) {
    perror("Failed to open Xillybus device file(s)");
    exit(-1);
  }
 
  if (fdffw < 0) {
    perror("Failed to open Xillybus device file(s)");
    exit(-1);
  }
 
  srcReal = cvLoadImage("fftreal.jpg", CV_LOAD_IMAGE_GRAYSCALE);
  srcImag = cvLoadImage("fftimag.jpg", CV_LOAD_IMAGE_GRAYSCALE);

  result = cvCreateImage(cvSize(512, 512), IPL_DEPTH_8U, 1);

  for(y = 0; y < WIDTH; y++) {
  
  //  TxtLoad("freal.txt", inR, y);
  //  TxtLoad("fimag.txt", inI, y);

    for(x = 0; x < WIDTH; x++) {
      pix[x] = srcReal->imageData[x*WIDTH+ y];
      if(pix[x] >= 128) {
        inR[x] = (short)(pix[x]) - 256;
      } else {
        inR[x] = (short)(pix[x]);
      } 
      pix[x] = srcImag->imageData[x*WIDTH+ y];
      if(pix[x] >= 128) {
        inI[x] = (short)(pix[x]) - 256;
      } else {
        inI[x] = (short)(pix[x]);
      }
      inR[x] = inR[x] << 2;
      inI[x] = inI[x] << 2;

//      if( x == 100 ) printf("inI[%d]= %d, pix= %d\n", x, inI[x], pix[x]);
    }

    for(i = 0; i < WAVEN; i++) {
      buf[i*4+0] = (unsigned char)(0xFF & inI[i]);
      buf[i*4+1] = (unsigned char)(0xFF & (inI[i] >> 8));
      buf[i*4+2] = (unsigned char)(0xFF & inR[i]);
      buf[i*4+3] = (unsigned char)(0xFF & (inR[i] >> 8));
//      if(i == 256) printf("inR[%d] = %d\n", i, inR[i]);
    }

    allwrite(fdffw, buf, WAVEN*4);
    //sleep(1);
    _wait(1000000);
  
    FifoCtrlWrite(fdw, 0x09); // Test mode, Test reset
    FifoCtrlWrite(fdw, 0x03); // Test mode, Test read
    //sleep(1);
    _wait(100000);
    FifoCtrlWrite(fdw, 0x11); // Test mode, FFT start
    //sleep(1);
    _wait(100000);
    FifoCtrlWrite(fdw, 0x05); // Test mode, Test write
    //sleep(1);
    _wait(100000);
    FifoCtrlWrite(fdw, 0);
  
    allread(fdffr, buf, WAVEN*4);
  
    for(i = 0; i < WAVEN; i++) {
      tmpshort = buf[i*4+0];
      tmpshort += (buf[i*4+1] << 8);
      fImag[i] = tmpshort;
      tmplong = tmpshort * tmpshort;
  
      tmpshort = (buf[i*4+2] << 0);
      tmpshort += (buf[i*4+3] << 8);
      fReal[i] = tmpshort;
      tmplong += (tmpshort * tmpshort);
  
      tmpdouble = sqrt((double)(tmplong));
      if(tmpdouble < 32767.1) fAbs[i] = (short)(tmpdouble);
      else fAbs[i] = 32767;

//      printf("buf[%d] = %d\n", i, fReal[i]);
    }
  
    for(x = 0; x < WIDTH; x++) {
      pix[x] = (unsigned char)(fAbs[x] >> 8);
      result->imageData[y*WIDTH+ x] = pix[x];
   //   printf("pix[%d]= %d\n", x, pix[x]);
    }

//    TxtSave("fabs.txt", fAbs, y);
//    TxtSave("freal.txt", fReal, y);
//    TxtSave("fimag.txt", fImag, y);
  }

  close(fdffr);
 
  cvSaveImage("fftres.jpg", result, 0);

  return 0;
}

void allread(int fd, unsigned char *buf, int len) {
  int received = 0;
  int rc;

  while (received < len) {
    rc = read(fd, buf + received, len - received);

    if ((rc < 0) && (errno == EINTR))
      continue;

    if (rc < 0) {
      perror("allread() failed to read");
      exit(1);
    }

    if (rc == 0) {
      fprintf(stderr, "Reached read EOF (?!)\n");
      exit(1);
    }

    received += rc;
  }
}

void TxtSave(char *fn, short *iW, int lineno)
{
  int i;
  char charline[40];
  FILE *fp;

  if(lineno == 0) fp = fopen( fn, "w" );
  else fp = fopen( fn, "a" );
  if( fp == NULL ){
    printf( "%s cannot be opened.\n", fn );
    exit(1);
  }

//  fseek( fp, 0L, SEEK_SET );
  for( i = 0; i < WAVEN; i++) {
    sprintf(charline, "%d\n", iW[i]);
    fputs(charline, fp);
//    printf( "%d\n", iW[i] );
  }
  fclose( fp );
}

void FifoCtrlWrite(int fd, unsigned char enb) {
  if (lseek(fd, 0, SEEK_SET) < 0) {
    perror("Failed to seek");
    exit(1);
  }
  allwrite(fd, &enb, 1);
}

void allwrite(int fd, unsigned char *buf, int len) {
  int sent = 0;
  int rc;

  while (sent < len) {
    rc = write(fd, buf + sent, len - sent);

    if ((rc < 0) && (errno == EINTR))
      continue;

    if (rc < 0) {
      perror("allwrite() failed to write");
      exit(1);
    }

    if (rc == 0) {
      fprintf(stderr, "Reached write EOF (?!)\n");
      exit(1);
    }

    sent += rc;
  }
}

void TxtLoad(char *fn, short *iW, int pixno)
{
  int datacount = 0;
  char charline[40];
  FILE *fp;
  int linecount = 0;

  fp = fopen( fn, "r" );
  if( fp == NULL ){
    printf( "%s cannot be opened.\n", fn );
    exit(1);
  }

  fseek( fp, 0L, SEEK_SET );
  while( fgets(charline, 40, fp) != NULL ) {
    if(linecount % WIDTH == pixno) {
      iW[datacount] = atoi(charline);// >> 8;
      iW[datacount] /= 8;
//      if(datacount == 0) {
//        printf( "%d  %d  %d\n", pixno, datacount, iW[datacount] );
//      }
      datacount++;
    }
    linecount++;

    if(datacount == WAVEN) {
      break;
    }
  }
  fclose( fp );
}


