-- The address generator of SRAM
-- by Toshio Iwata at digitalfilter.com 2009/06/12

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_unsigned.all;

entity AdrsGen is
port (
  CLK : in std_logic;
  RST_N : in std_logic;
  FFT_START : in std_logic; 
  FFT_SEL : out std_logic; 
  FFT_END : out std_logic;
  PHASE_K : out std_logic;
  PHASE_I : out std_logic;
  FFT_ADDR : out std_logic_vector(9 downto 0); 
  TRI_ADDR : out std_logic_vector(9 downto 0) ); 
end AdrsGen;

architecture RTL of AdrsGen is

signal count_k : unsigned(10 downto 0);
signal count_k_next : unsigned(10 downto 0);
signal count_k_max : std_logic;
signal count_j : unsigned(10 downto 0);
signal count_j_next : unsigned(10 downto 0);
signal count_j_max : std_logic;
signal count_i : unsigned(3 downto 0);
signal count_i_max : std_logic;
signal hclk : std_logic;
signal fft_start_dly : std_logic;
signal fft_start_edge : std_logic;
signal fft_sel_sig : std_logic;
signal fft_end_sig : std_logic;
signal mm : unsigned(10 downto 0);
signal hh : unsigned(9 downto 0);
	signal addr_k : unsigned(9 downto 0);
	signal addr_kp : unsigned(9 downto 0);
	signal tribase : unsigned(9 downto 0);
	signal count_cos : unsigned(9 downto 0);
	signal count_sin : unsigned(9 downto 0);
	signal count_j_sft0 : unsigned(9 downto 0);
	signal count_j_sft1 : unsigned(9 downto 0);
	signal count_j_sft2 : unsigned(9 downto 0);
	signal count_j_sft3 : unsigned(9 downto 0);
	signal count_j_sft4 : unsigned(9 downto 0);
	signal count_j_sft5 : unsigned(9 downto 0);
	signal count_j_sft6 : unsigned(9 downto 0);
	signal count_j_sft7 : unsigned(9 downto 0);
	signal count_j_sft8 : unsigned(9 downto 0);
	signal count_j_sft9 : unsigned(9 downto 0);
	signal intervalcount : unsigned(2 downto 0);

begin
-------------------------------------------------------------------
	count_j_sft8 <= "0" & count_j( 8 downto 0 );
	count_j_sft7 <= "0" & count_j( 7 downto 0 ) & "0";
	count_j_sft6 <= "0" & count_j( 6 downto 0 ) & "00";
	count_j_sft5 <= "0" & count_j( 5 downto 0 ) & "000";
	count_j_sft4 <= "0" & count_j( 4 downto 0 ) & "0000";
	count_j_sft3 <= "0" & count_j( 3 downto 0 ) & "00000";
	count_j_sft2 <= "0" & count_j( 2 downto 0 ) & "000000";
	count_j_sft1 <= "0" & count_j( 1 downto 0 ) & "0000000";
	count_j_sft0 <= "0" & count_j( 0 ) & "00000000"; -- iwata

	tribase <= count_j_sft0 when ( count_i = 0 ) else 
		count_j_sft1 when ( count_i = 1 ) else 
		count_j_sft2 when ( count_i = 2 ) else 
		count_j_sft3 when ( count_i = 3 ) else 
		count_j_sft4 when ( count_i = 4 ) else 
		count_j_sft5 when ( count_i = 5 ) else 
		count_j_sft6 when ( count_i = 6 ) else 
		count_j_sft7 when ( count_i = 7 ) else 
		count_j_sft8 when ( count_i = 8 ) else 
		(others => '0' );

	count_cos <= tribase + 0;
	count_sin <= tribase + 256; -- iwata

-------------------------------------------------------------------
	gen_TRI_ADDR : process( RST_N, CLK )
	begin
		if( RST_N = '0' ) then
			TRI_ADDR <= (others => '0');
		elsif( CLK'event and CLK = '1' ) then
			if( hclk = '0' ) then
				for i in 9 downto 0 loop
					TRI_ADDR(i) <= count_cos(i);
				end loop;
			else 
				for i in 9 downto 0 loop
					TRI_ADDR(i) <= count_sin(i);
				end loop;
			end if;
		end if;
	end process;

-------------------------------------------------------------------
  PHASE_K <= hclk;
  
-------------------------------------------------------------------
	addr_k <= count_k(9 downto 0);
	addr_kp <= count_k(9 downto 0) + hh;

-------------------------------------------------------------------
	gen_addr : process( RST_N, CLK )
	begin
		if( RST_N = '0' ) then
			FFT_ADDR <= (others => '0');
		elsif( CLK'event and CLK = '1' ) then
			if( hclk = '1' ) then
				for i in 9 downto 0 loop
					FFT_ADDR(i) <= addr_k(i);
				end loop;
			else 
				for i in 9 downto 0 loop
					FFT_ADDR(i) <= addr_kp(i);
				end loop;
			end if;
		end if;
	end process;

-------------------------------------------------------------------
	mm <= 	"00000000010" when count_i = 0 else 
				"00000000100" when count_i = 1 else
				"00000001000" when count_i = 2 else
				"00000010000" when count_i = 3 else
				"00000100000" when count_i = 4 else
				"00001000000" when count_i = 5 else
				"00010000000" when count_i = 6 else 
				"00100000000" when count_i = 7 else 
				"01000000000" when count_i = 8 else 
				"10000000000" when count_i = 9 else 
				(others => '0');

	hh <= 	"0000000001" when count_i = 0 else 
				"0000000010" when count_i = 1 else
				"0000000100" when count_i = 2 else
				"0000001000" when count_i = 3 else
				"0000010000" when count_i = 4 else
				"0000100000" when count_i = 5 else
				"0001000000" when count_i = 6 else 
				"0010000000" when count_i = 7 else 
				"0100000000" when count_i = 8 else 
				"1000000000" when count_i = 9 else 
				(others => '0');

-------------------------------------------------------------------
gen_count_k : process( fft_start_edge, RST_N, CLK )
begin
  if( RST_N = '0' or fft_start_edge = '1') then
    count_k <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if( hclk = '1' and fft_sel_sig = '1' ) then
      if ((count_k_max = '1' and count_j_max = '1') or intervalcount /= 0) then -- clear
        count_k <= (others => '0');
      elsif (count_k_max = '1' and count_j_max = '0') then -- load
        count_k <= count_j_next;
      else -- increment by mm
        count_k <= count_k + mm;
      end if;
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_intervalcont : process( fft_start_edge, RST_N, CLK )
begin
  if( RST_N = '0' or fft_start_edge = '1') then
    intervalcount <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if( hclk = '1' and fft_sel_sig = '1' ) then
      if ( count_k_max = '1' and count_j_max = '1') then -- load
        intervalcount <= "011";
      elsif(intervalcount /= 0) then -- decrement by 1
        intervalcount <= intervalcount - 1;
      end if;
    end if;
  end if;
end process;

-------------------------------------------------------------------
  count_k_next <= count_k + mm;
  count_k_max <= '1' when count_k_next >= 512 else '0'; -- iwata
  
-------------------------------------------------------------------
gen_count_j : process( fft_start_edge, RST_N, CLK )
begin
  if( RST_N = '0' or fft_start_edge = '1' ) then
    count_j <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if( hclk = '1' and fft_sel_sig = '1' ) then
      if ((count_k_max = '1' and count_j_max = '1') or intervalcount /= 0) then -- clear
        count_j <= (others => '0');
      elsif ( count_k_max = '1' and count_j_max = '0') then -- increment by 1
        count_j <= count_j + 1;
      end if;
    end if;
  end if;
end process;

-------------------------------------------------------------------
  count_j_next <= count_j + 1;
  count_j_max <= '1' when count_j_next = hh else '0';

-------------------------------------------------------------------
gen_count_i : process( fft_start_edge, RST_N, CLK )
begin
  if( RST_N = '0' or fft_start_edge = '1') then
    count_i <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if( hclk = '1' and fft_sel_sig = '1' ) then
      if ( count_k_max = '1' and count_j_max = '1' and count_i_max = '1' ) then -- load
        count_i <= (others => '0'); -- iwata
      elsif ( count_k_max = '1' and count_j_max = '1' and count_i_max = '0' ) then -- increment by 1
        count_i <= count_i + 1;
      end if;
    end if;
  end if;
end process;

-------------------------------------------------------------------
  count_i_max <= '1' when count_i = 8 else '0'; -- iwata
  FFT_END <= fft_end_sig;
  FFT_SEL <= fft_sel_sig;

-------------------------------------------------------------------
gen_fft_end_sig : process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    fft_end_sig <= '0';
  elsif( CLK'event and CLK = '1' ) then
    if(count_i_max = '1' and count_j_max = '1' and hclk = '1') then
      fft_end_sig <= '1';
    else 
      fft_end_sig <= '0';
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_fft_start_dly : process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    fft_start_dly <= '0';
  elsif( CLK'event and CLK = '1' ) then
    fft_start_dly <= FFT_START;
  end if;
end process;

-------------------------------------------------------------------
  fft_start_edge <= FFT_START and not fft_start_dly;

-------------------------------------------------------------------
gen_fft_sel_sig : process( fft_start_edge, RST_N, CLK )
begin
  if( RST_N = '0') then
    fft_sel_sig <= '0';
  elsif( CLK'event and CLK = '1' ) then
    if(FFT_START = '1' and fft_start_dly = '0') then
      fft_sel_sig <= '1';
    elsif(fft_end_sig = '1') then
      fft_sel_sig <= '0';
    end if;
  end if;
end process;

-------------------------------------------------------------------
gen_hclk : process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    hclk <= '0';
  elsif( CLK'event and CLK = '1' ) then
    hclk <= not hclk;
  end if;
end process;

-------------------------------------------------------------------
gen_phase_i : process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    PHASE_I <= '0';
  elsif( CLK'event and CLK = '1' ) then
    PHASE_I <= count_i(0);
  end if;
end process;
  
end RTL;

