-- Timing Generator for A/D, D/A converter etc.
-- by Toshio Iwata at digitalfilter.com 2009/06/12

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_unsigned.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity TimingGen is
port( 	
  CLK : in std_logic;
  RST_N : in std_logic;
  CAP_START : in std_logic;
  CAP_SEL : out std_logic;
  ALRCLK : Out std_logic;
  ABCLK : Out std_logic;
  AMCLK : Out std_logic;
  ADDRADC : Out std_logic_vector(9 downto 0) ); 
end TimingGen;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of TimingGen is

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
  signal counter512 : unsigned(8 downto 0);
  signal counter_fs : unsigned(9 downto 0);
	signal trig_dly1 : std_logic;
	signal trig_dly2 : std_logic;
	signal trig_dly3 : std_logic;
	signal trig_edge : std_logic;

begin
-------------------------------------------------------------------
	gen_trig_dly : process( RST_N, CLK )
	begin
		if( RST_N = '0' ) then
			trig_dly1 <= '0';
			trig_dly2 <= '0';
			trig_dly3 <= '0';
		elsif( CLK'event and CLK = '1' ) then
			trig_dly1 <= CAP_START;
			trig_dly2 <= trig_dly1;
			trig_dly3 <= trig_dly2;
		end if;
	end process;

-------------------------------------------------------------------
	trig_edge <= trig_dly1 and not trig_dly2;

-------------------------------------------------------------------
gen_counter512 : process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    counter512  <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    counter512 <= counter512 + 1; -- up counter
  end if;
end process;

-------------------------------------------------------------------
  AMCLK <= counter512(0);
  ABCLK <= counter512(2);
  ALRCLK <= counter512(8);

-------------------------------------------------------------------
gen_counter_fs : process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    counter_fs  <= (others => '0');
  elsif( CLK'event and CLK = '1' ) then
    if(trig_edge = '1') then 
      counter_fs <= (others => '0');
    elsif(counter_fs /= 1023 and counter512 = 511) then 
      counter_fs <= counter_fs + 1; -- up counter
    end if;
  end if;
end process;

----------------------------------------------------
process( counter_fs )
begin
  for i in 9 downto 0 loop
    ADDRADC(9-i) <= counter_fs(i);
  end loop;
end process;

----------------------------------------------------
gen_CAP_SEL : process( RST_N, CLK )
begin
  if( RST_N = '0' ) then
    CAP_SEL  <= '0';
  elsif( CLK'event and CLK = '1' ) then
    if(trig_edge = '1') then 
      CAP_SEL <= '1';
    elsif(counter_fs = 1023 and counter512 = 511) then 
      CAP_SEL <= '0';
    end if;
  end if;
end process;

end RTL;

