--------------------------------------------------
-- BUTTERFLY.vhd
-- Butterfly calculation
-- by Toshio Iwata at digitalfilter.com 2009/06/12

library ieee ;
  use ieee.std_logic_1164.all ;
  use ieee.std_logic_signed.all ;

--------------------------------------------------------------------------------
-- Entity                                                                       
--------------------------------------------------------------------------------

entity BUTTERFLY is
  port ( 
    MCLK         : in    std_logic                           ; 
    RST_X        : in    std_logic                           ; 
    PHASE_K       : in    std_logic                           ; 
    TRI_RD_DATA  : in    std_logic_vector(  19 downto   0 )  ;
    FFT_REAL_RD_DATA : in    std_logic_vector(  15 downto   0 )  ; 
    FFT_IMAG_RD_DATA : in    std_logic_vector(  15 downto   0 )  ; 
    FFT_REAL_WR_DATA : out   std_logic_vector(  15 downto   0 )  ; 
    FFT_IMAG_WR_DATA : out   std_logic_vector(  15 downto   0 )    
  ) ;
end BUTTERFLY ;


--------------------------------------------------------------------------------
-- Architecture                                                                 
--------------------------------------------------------------------------------

architecture RTL of BUTTERFLY is

  ------------------------------------------------------------
  -- Constant Declaration 
  ------------------------------------------------------------

  ------------------------------------------------------------
  -- Signal Declaration (Register Resource : Total 553 FF) 
  ------------------------------------------------------------
  signal   r_phase_k_dly1        : std_logic                           ; 
  signal   r_phase_k_dly2        : std_logic                           ; 
  signal   r_phase_k_dly3        : std_logic                           ; 
  signal   r_phase_k_dly4        : std_logic                           ; 
  signal   r_phase_k_dly5        : std_logic                           ; 
  signal   r_tri_rd_data_dly1   : std_logic_vector(  19 downto   0 )  ; 
  signal   r_real_rd_k          : std_logic_vector(  15 downto   0 )  ; 
  signal   r_real_rd_k_dly1     : std_logic_vector(  15 downto   0 )  ; 
  signal   r_real_rd_kp         : std_logic_vector(  15 downto   0 )  ; 
  signal   r_mul_real_1         : std_logic_vector(  35 downto   0 )  ; 
  signal   r_mul_real_2         : std_logic_vector(  35 downto   0 )  ; 
  signal   r_v1                 : std_logic_vector(  15 downto   0 )  ; 
  signal   r_real_wr_dly1       : std_logic_vector(  15 downto   0 )  ; 
  signal   r_imag_rd_k          : std_logic_vector(  15 downto   0 )  ; 
  signal   r_imag_rd_k_dly1     : std_logic_vector(  15 downto   0 )  ; 
  signal   r_imag_rd_kp         : std_logic_vector(  15 downto   0 )  ; 
  signal   r_mul_imag_1         : std_logic_vector(  35 downto   0 )  ; 
  signal   r_mul_imag_2         : std_logic_vector(  35 downto   0 )  ; 
  signal   r_v2                 : std_logic_vector(  15 downto   0 )  ; 
  signal   r_imag_wr_dly1       : std_logic_vector(  15 downto   0 )  ; 

  ------------------------------------------------------------
  -- Signal Declaration (Non Register Resource)         
  ------------------------------------------------------------
  signal   s_multiplier_a       : std_logic_vector(  35 downto   0 )  ; 
  signal   s_adder_real         : std_logic_vector(  35 downto   0 )  ; 
  signal   s_real_wr_kp         : std_logic_vector(  15 downto   0 )  ;
  signal   s_real_wr_k          : std_logic_vector(  15 downto   0 )  ; 
  signal   s_real_wr            : std_logic_vector(  15 downto   0 )  ; 
  signal   s_multiplier_b       : std_logic_vector(  35 downto   0 )  ; 
  signal   s_adder_imag         : std_logic_vector(  35 downto   0 )  ; 
  signal   s_imag_wr_kp         : std_logic_vector(  15 downto   0 )  ; 
  signal   s_imag_wr_k          : std_logic_vector(  15 downto   0 )  ; 
  signal   s_imag_wr            : std_logic_vector(  15 downto   0 )  ; 

begin

  ------------------------------------------------------------
  --  An enable signal for DF/F 
  ------------------------------------------------------------
  phase_k_dly1_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_phase_k_dly1 <= '0' ;
    elsif ( MCLK'event and MCLK = '1' ) then
        r_phase_k_dly1 <= PHASE_K ;
    end if ;
  end process phase_k_dly1_PROC ;

  ------------------------------------------------------------
  --  An enable signal for DF/F 
  ------------------------------------------------------------
  phase_k_dly2_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_phase_k_dly2 <= '0' ;
    elsif ( MCLK'event and MCLK = '1' ) then
        r_phase_k_dly2 <= r_phase_k_dly1 ;
    end if ;
  end process phase_k_dly2_PROC ;

  ------------------------------------------------------------
  --  An enable signal for DF/F 
  ------------------------------------------------------------
  phase_k_dly3_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_phase_k_dly3 <= '0' ;
    elsif ( MCLK'event and MCLK = '1' ) then
        r_phase_k_dly3 <= r_phase_k_dly2 ;
    end if ;
  end process phase_k_dly3_PROC ;

  ------------------------------------------------------------
  --  An enable signal for DF/F 
  ------------------------------------------------------------
  phase_k_dly4_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_phase_k_dly4 <= '0' ;
    elsif ( MCLK'event and MCLK = '1' ) then
        r_phase_k_dly4 <= r_phase_k_dly3 ;
    end if ;
  end process phase_k_dly4_PROC ;

  ------------------------------------------------------------
  --  An enable signal for DF/F 
  ------------------------------------------------------------
  phase_k_dly5_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_phase_k_dly5 <= '0' ;
    elsif ( MCLK'event and MCLK = '1' ) then
        r_phase_k_dly5 <= r_phase_k_dly4 ;
    end if ;
  end process phase_k_dly5_PROC ;

  ------------------------------------------------------------
  --  Delay Triangle Data 
  ------------------------------------------------------------
  tri_rd_data_dly1_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_tri_rd_data_dly1 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
        r_tri_rd_data_dly1 <= TRI_RD_DATA ;
    end if ;
  end process tri_rd_data_dly1_PROC ;

  ------------------------------------------------------------
  --  for Real Parts 
  ------------------------------------------------------------
  ------------------------------------------------------------
  --  Pick up "Real[k]" 
  ------------------------------------------------------------
  real_rd_k_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_real_rd_k <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
        if ( r_phase_k_dly2 = '1' ) then
          r_real_rd_k <= FFT_REAL_RD_DATA ;
        end if ;
    end if ;
  end process real_rd_k_PROC ;

  ------------------------------------------------------------
  --  Delay "Real[k]" 
  ------------------------------------------------------------
  real_rd_k_dly1_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_real_rd_k_dly1 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
        if ( r_phase_k_dly4 = '1' ) then
          r_real_rd_k_dly1 <= r_real_rd_k ;
        end if ;
    end if ;
  end process real_rd_k_dly1_PROC ;

  ------------------------------------------------------------
  --  Pick up "Real[kp]" 
  ------------------------------------------------------------
  real_rd_kp_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_real_rd_kp <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
        if ( r_phase_k_dly1 = '1' ) then
          r_real_rd_kp <= FFT_REAL_RD_DATA ;
        end if ;
    end if ;
  end process real_rd_kp_PROC ;

  ------------------------------------------------------------
  --  Multiplier-A  
  ------------------------------------------------------------
  s_multiplier_a <= r_real_rd_kp( 15 downto  0 ) * r_tri_rd_data_dly1( 19 downto  0 ) ;
  
  ------------------------------------------------------------
  --  Real[kp] * cos 
  ------------------------------------------------------------
  mul_real_1_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_mul_real_1 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
        if ( r_phase_k_dly2 = '1' ) then
          r_mul_real_1 <= s_multiplier_a ;
        end if ;
    end if ;
  end process mul_real_1_PROC ;

  ------------------------------------------------------------
  --  Real[kp] * sin 
  ------------------------------------------------------------
  mul_real_2_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_mul_real_2 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
        if ( r_phase_k_dly3 = '1' ) then
          r_mul_real_2 <= s_multiplier_a ;
        end if ;
    end if ;
  end process mul_real_2_PROC ;

  ------------------------------------------------------------
  --  Adder for real parts 
  ------------------------------------------------------------
  s_adder_real <= r_mul_real_1( 35 downto  0 ) - r_mul_imag_2( 35 downto  0 ) ;
  
  ------------------------------------------------------------
  --  Generate "v1" 
  ------------------------------------------------------------
  v1_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_v1 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
        if ( r_phase_k_dly4 = '1' ) then
          r_v1 <= s_adder_real( 34 downto 19 ) ;
        end if ;
    end if ;
  end process v1_PROC ;

  ------------------------------------------------------------
  --  Subtractor to generate new "Real[kp]" 
  ------------------------------------------------------------
  s_real_wr_kp <= r_real_rd_k_dly1( 15 downto  0 ) - r_v1( 15 downto  0 ) ;
  
  ------------------------------------------------------------
  --  Subtractor to generate new "Real[k]" 
  ------------------------------------------------------------
  s_real_wr_k <= r_real_rd_k_dly1( 15 downto  0 ) + r_v1( 15 downto  0 ) ;
  
  ------------------------------------------------------------
  --  New real parts 
  ------------------------------------------------------------
  s_real_wr <=     s_real_wr_kp  when ( r_phase_k_dly5 = '1' ) else
                   s_real_wr_k ;

  
  ------------------------------------------------------------
  --  Delay real parts 
  ------------------------------------------------------------
  real_wr_dly1_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_real_wr_dly1 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
        r_real_wr_dly1 <= s_real_wr ;
    end if;
   end process real_wr_dly1_PROC ;

  --  Output real parts  [ Assign @35(fft_butterfly_arch) ] 
  FFT_REAL_WR_DATA <= r_real_wr_dly1( 15 downto  0 ) ;
  
  ------------------------------------------------------------
  --  for Imaginary Parts 
  ------------------------------------------------------------
  ------------------------------------------------------------
  --  Pick up "Real[k]" 
  ------------------------------------------------------------
  imag_rd_k_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_imag_rd_k <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
      if ( r_phase_k_dly2 = '1' ) then
           r_imag_rd_k <= FFT_IMAG_RD_DATA ;
      end if;
    end if ;
  end process imag_rd_k_PROC ;

  ------------------------------------------------------------
  --  Delay "Real[k]" 
  ------------------------------------------------------------
  imag_rd_k_dly1_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_imag_rd_k_dly1 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
        if ( r_phase_k_dly4 = '1' ) then
          r_imag_rd_k_dly1 <= r_imag_rd_k ;
        end if ;
    end if ;
  end process imag_rd_k_dly1_PROC ;

  ------------------------------------------------------------
  --  Pick up "Real[kp]" 
  ------------------------------------------------------------
  imag_rd_kp_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_imag_rd_kp <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
        if ( r_phase_k_dly1 = '1' ) then
          r_imag_rd_kp <= FFT_IMAG_RD_DATA ;
        end if ;
    end if ;
  end process imag_rd_kp_PROC ;

  ------------------------------------------------------------
  --  Multiplier-B  
  ------------------------------------------------------------
  s_multiplier_b <= r_imag_rd_kp( 15 downto  0 ) * r_tri_rd_data_dly1( 19 downto  0 ) ;
  
  ------------------------------------------------------------
  --  Imag[kp] * cos 
  ------------------------------------------------------------
  mul_imag_1_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_mul_imag_1 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
        if ( r_phase_k_dly2 = '1' ) then
          r_mul_imag_1 <= s_multiplier_b ;
        end if ;
    end if ;
  end process mul_imag_1_PROC ;

  ------------------------------------------------------------
  --  Imag[kp] * sin 
  ------------------------------------------------------------
  mul_imag_2_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_mul_imag_2 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
        if ( r_phase_k_dly3 = '1' ) then
          r_mul_imag_2 <= s_multiplier_b ;
        end if ;
    end if ;
  end process mul_imag_2_PROC ;

  ------------------------------------------------------------
  --  Adder for imag parts  
  ------------------------------------------------------------
  s_adder_imag <= r_mul_imag_1( 35 downto  0 ) + r_mul_real_2( 35 downto  0 ) ;
  
  ------------------------------------------------------------
  --  Generate "v2" 
  --  [ En-FF x 32 @67(fft_butterfly_arch) ]
  ------------------------------------------------------------
  v2_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_v2 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
        if ( r_phase_k_dly4 = '1' ) then
          r_v2 <= s_adder_imag( 34 downto 19 ) ;
        end if ;
    end if ;
  end process v2_PROC ;

  ------------------------------------------------------------
  --  Subtractor to generate new "Imag[kp]" 
  ------------------------------------------------------------
  s_imag_wr_kp <= r_imag_rd_k_dly1( 15 downto  0 ) - r_v2( 15 downto  0 ) ;
  
  ------------------------------------------------------------
  --  Subtractor to generate new "Imag[k]"  
  ------------------------------------------------------------
  s_imag_wr_k <= r_imag_rd_k_dly1( 15 downto  0 ) + r_v2( 15 downto  0 ) ;
  
  ------------------------------------------------------------
  --  New imagnary parts 
  ------------------------------------------------------------
  s_imag_wr <=     s_imag_wr_kp  when ( r_phase_k_dly5 = '1' ) else
                   s_imag_wr_k ;

  
  ------------------------------------------------------------
  --  Delay imaginary parts 
  ------------------------------------------------------------
  imag_wr_dly1_PROC : process ( RST_X , MCLK )
  begin
    if ( RST_X = '0' ) then
      r_imag_wr_dly1 <= (others => '0') ;
    elsif ( MCLK'event and MCLK = '1' ) then
         r_imag_wr_dly1 <= s_imag_wr ;
      end if ;
  end process imag_wr_dly1_PROC ;

  ------------------------------------------------------------
  --  Output imag parts 
  ------------------------------------------------------------
  FFT_IMAG_WR_DATA <= r_imag_wr_dly1( 15 downto  0 ) ;
  
end RTL ;

-- end of 'butterfly.vhd'
