#include <cv.h>
#include <highgui.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdint.h>
#include <string.h>

#define SCALEX 640
#define SCALEY 320
#define OFTX 40
#define OFTY 20
#define STEPX 4
#define WAVEN 512

void _wait(loop_count)
int loop_count;
{
  volatile int sum, data;
  sum = 0;
  for(data = 0; data < loop_count; data++) {
    sum = (data << 8);
  }
  return;
}

void DrawScale(CvArr* img);
void FifoCtrlWrite(int fd, unsigned char enb);
void allread(int fd, unsigned char *buf, int len);
void allwrite(int fd, unsigned char *buf, int len);

int main (int argc, char **argv)
{
  int i;
  IplImage *img;
  int val, valp1;
  int fdw, fdffr, fdffw;
  unsigned char buf[WAVEN*4];
  short fReal[WAVEN];
  short fImag[WAVEN];
  long fAbs[WAVEN];
  short tmpshort;
  long tmplong;
//  long long tmplonglong;

  img = cvCreateImage (cvSize (720, 360), 8, 1);

  cvSet (img, cvScalarAll (255), 0);

  CvScalar color = CV_RGB( rand()&255, rand()&255, rand()&255 );
  color = CV_RGB(0, 255, 0);

  fdw = open("/dev/xillybus_mem_8", O_WRONLY);
  fdffr = open("/dev/xillybus_read_32", O_RDONLY);
  if (fdffr < 0 || fdw < 0) {
    perror("Failed to open Xillybus device file(s)");
    exit(-1);
  }

  while (1) {

    FifoCtrlWrite(fdw, 0x08); // Reset
    FifoCtrlWrite(fdw, 0x02); // Test Read
    _wait(800000);
    FifoCtrlWrite(fdw, 0x10); // FFT Start
    _wait(50000);
    FifoCtrlWrite(fdw, 0x04); // Test Write
    _wait(100000);
    FifoCtrlWrite(fdw, 0);

    allread(fdffr, buf, WAVEN*4);

    for(i = 0; i < WAVEN; i++) {
      tmpshort = buf[i*4+0];
      tmpshort += (buf[i*4+1] << 8);
      fImag[i] = tmpshort;
      tmplong = tmpshort * tmpshort;

      tmpshort = (buf[i*4+2] << 0);
      tmpshort += (buf[i*4+3] << 8);
      fReal[i] = tmpshort;
      tmplong += (tmpshort * tmpshort);

      fAbs[i] = (long)(sqrt((double)(tmplong)));
      printf("buf[%d] = %ld\n", i, fAbs[i]);
    }

    cvRectangle (img,
               cvPoint (OFTX, OFTY),
               cvPoint (OFTX+SCALEX, OFTY+SCALEY), 
               CV_RGB(255,255,255), CV_FILLED, 8, 0);

    DrawScale(img);

    for(i = 0; i < 500; i++) {
      if(STEPX*i < SCALEX) {
        val = fAbs[i] >> 8;
        valp1 = fAbs[i+1] >> 8;
        cvLine( img, cvPoint(OFTX+STEPX*i, OFTY + SCALEY*3/4 - val),
                     cvPoint(OFTX+STEPX*i + STEPX, OFTY + SCALEY*3/4 - valp1),
                     color, 1, 8, 0);
      }
    }

    cvNamedWindow ("Waveform", CV_WINDOW_AUTOSIZE);
    cvShowImage ("Waveform", img);
    if(cvWaitKey(1)>=0) break;
  

  }

  cvDestroyWindow ("Waveform");
  cvReleaseImage (&img);

  close(fdffr);

  return 0;
}

void DrawScale(CvArr* img) {
  int fullperiod = 15;
  int i;

  CvFont font;
  int font_face[] = {
    CV_FONT_HERSHEY_SIMPLEX,
    CV_FONT_HERSHEY_PLAIN,
    CV_FONT_HERSHEY_DUPLEX,
    CV_FONT_HERSHEY_COMPLEX,
    CV_FONT_HERSHEY_TRIPLEX,
    CV_FONT_HERSHEY_COMPLEX_SMALL,
    CV_FONT_HERSHEY_SCRIPT_SIMPLEX,
    CV_FONT_HERSHEY_SCRIPT_COMPLEX
  };
  char tmpchar[16];

  cvRectangle (img,
               cvPoint (OFTX, OFTY),
               cvPoint (OFTX+SCALEX, OFTY+SCALEY),
               cvScalarAll (0), 1, 8, 0);

  cvInitFont(&font, font_face[0], 0.5, 0.5, 0, 1, 8);

  sprintf( tmpchar, "0Hz");
  cvPutText(img, tmpchar, cvPoint(OFTX + 3, OFTY + SCALEY - 10) , &font, CV_RGB(0, 0, 0));
  sprintf( tmpchar, "%dHz", fullperiod*1/3 );
  cvPutText(img, tmpchar, cvPoint(OFTX + SCALEX/3 - 15, OFTY + SCALEY - 10) , &font, CV_RGB(0, 0, 0));
  sprintf( tmpchar, "%dHz", fullperiod*2/3 );
  cvPutText(img, tmpchar, cvPoint(OFTX + SCALEX*2/3 - 15, OFTY + SCALEY - 10) , &font, CV_RGB(0, 0, 0));
  sprintf( tmpchar, "%dHz", fullperiod*3/3 );
  cvPutText(img, tmpchar, cvPoint(OFTX + SCALEX - 35, OFTY + SCALEY - 10) , &font, CV_RGB(0, 0, 0));

  for(i = 1; i <= 14; i++) {
    cvLine( img, cvPoint(OFTX + SCALEX*i/15, OFTY + SCALEY),
                   cvPoint(OFTX + SCALEX*i/15, OFTY),
                   CV_RGB(230, 230, 230), 1, 8, 0);
  }

  for(i = 1; i <= 7; i++) {
    cvLine( img, cvPoint(OFTX, OFTY + SCALEY*i/8),
                   cvPoint(OFTX + SCALEX, OFTY + SCALEY*i/8),
                   CV_RGB(230, 230, 230), 1, 8, 0);
  }

  cvLine( img, cvPoint(OFTX + SCALEX/3, OFTY + SCALEY),
                   cvPoint(OFTX + SCALEX/3, OFTY),
                   CV_RGB(200, 200, 200), 1, 8, 0);
  cvLine( img, cvPoint(OFTX + SCALEX*2/3, OFTY + SCALEY),
                   cvPoint(OFTX + SCALEX*2/3, OFTY),
                   CV_RGB(200, 200, 200), 1, 8, 0);
  cvLine( img, cvPoint(OFTX, OFTY + SCALEY*3/4),
                   cvPoint(OFTX + SCALEX, OFTY + SCALEY*3/4),
                   CV_RGB(200, 200, 200), 1, 8, 0);
}

void allread(int fd, unsigned char *buf, int len) {
  int received = 0;
  int rc;

  while (received < len) {
    rc = read(fd, buf + received, len - received);

    if ((rc < 0) && (errno == EINTR))
      continue;

    if (rc < 0) {
      perror("allread() failed to read");
      exit(1);
    }

    if (rc == 0) {
      fprintf(stderr, "Reached read EOF (?!)\n");
      exit(1);
    }

    received += rc;
  }
}

void allwrite(int fd, unsigned char *buf, int len) {
  int sent = 0;
  int rc;

  while (sent < len) {
    rc = write(fd, buf + sent, len - sent);

    if ((rc < 0) && (errno == EINTR))
      continue;

    if (rc < 0) {
      perror("allwrite() failed to write");
      exit(1);
    }

    if (rc == 0) {
      fprintf(stderr, "Reached write EOF (?!)\n");
      exit(1);
    }

    sent += rc;
  }
}

void FifoCtrlWrite(int fd, unsigned char enb) {
  if (lseek(fd, 0, SEEK_SET) < 0) {
    perror("Failed to seek");
    exit(1);
  }
  allwrite(fd, &enb, 1);
}

